{% extends theme('base.html') %}

{% block content %}
    <div id="modal-reset-otp" class="ui warning message">
        <form method="post" action="{{ request.url }}">
            <input type="hidden" name="csrf_token" value="{{ request.form.get("csrf_token") }}">
            <div class="ui icon header">
                <i class="key icon"></i>
                {% trans %}One-time passcode authentication reset{% endtrans %}
            </div>
            <div class="content">
                <p>
                    {% if user != edited_user %}
                        {% trans user_name=(edited_user.formatted_name or edited_user.identifier) %}
                            Are you sure you want to reset the authenticator application configuration for {{ user_name }}?
                        {% endtrans %}
                    {% else %}
                        {% trans %}
                            Are you sure you want to reset your authenticator application configuration?
                        {% endtrans %}
                    {% endif %}
                </p>
            </div>
            <div class="ui center aligned container">
                <div class="ui stackable buttons">
                    <a class="ui cancel button" href="{{ request.url }}">{% trans %}Cancel{% endtrans %}</a>
                    <button type="submit" name="action" value="reset-otp" class="ui red approve button">
                        {% trans %}Confirm{% endtrans %}
                    </button>
                </div>
            </div>
        </form>
    </div>
{% endblock %}
